@@ -4,7 +4,7 @@
  *	Utility routines shared by pg_dump and pg_restore
  *
  *
- * Portions Copyright (c) 1996-2014, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2018, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * src/bin/pg_dump/pg_backup_utils.c
@@ -13,8 +13,8 @@
  */
 #include "postgres_fe.h"
 
-#include "pg_backup_utils.h"
 #include "parallel.h"
+#include "pg_backup_utils.h"
 
 /* Globals exported by this file */
 const char *progname = NULL;
@@ -25,7 +25,7 @@ static struct
 {
 	on_exit_nicely_callback function;
 	void	   *arg;
-}	on_exit_nicely_list[MAX_ON_EXIT_NICELY];
+}			on_exit_nicely_list[MAX_ON_EXIT_NICELY];
 
 static int	on_exit_nicely_index;
 
@@ -144,8 +144,8 @@ exit_nicely(int code)
 	int			i;
 
 	for (i = on_exit_nicely_index - 1; i >= 0; i--)
-		(*on_exit_nicely_list[i].function) (code,
-											on_exit_nicely_list[i].arg);
+		on_exit_nicely_list[i].function(code,
+										on_exit_nicely_list[i].arg);
 
 #ifdef WIN32
 	if (parallel_init_done && GetCurrentThreadId() != mainThreadId)
