@@ -452,7 +452,17 @@ sub init
 	print $conf "log_statement = all\n";
 	print $conf "log_replication_commands = on\n";
 	print $conf "wal_retrieve_retry_interval = '500ms'\n";
-	print $conf "port = $port\n";
+
+	# If a setting tends to affect whether tests pass or fail, print it after
+	# TEMP_CONFIG.  Otherwise, print it before TEMP_CONFIG, thereby permitting
+	# overrides.  Settings that merely improve performance or ease debugging
+	# belong before TEMP_CONFIG.
+	print $conf TestLib::slurp_file($ENV{TEMP_CONFIG})
+	  if defined $ENV{TEMP_CONFIG};
+
+	# XXX Neutralize any stats_temp_directory in TEMP_CONFIG.  Nodes running
+	# concurrently must not share a stats_temp_directory.
+	print $conf "stats_temp_directory = 'pg_stat_tmp'\n";
 
 	if ($params{allows_streaming})
 	{
@@ -478,6 +488,7 @@ sub init
 		print $conf "max_wal_senders = 0\n";
 	}
 
+	print $conf "port = $port\n";
 	if ($use_tcp)
 	{
 		print $conf "unix_socket_directories = ''\n";
@@ -1044,14 +1055,15 @@ sub get_new_node
 		# This seems like a good idea on Unixen as well, even though we don't
 		# ask the postmaster to open a TCP port on Unix.  On Non-Linux,
 		# non-Windows kernels, binding to 127.0.0.1/24 addresses other than
-		# 127.0.0.1 fails with EADDRNOTAVAIL.
+		# 127.0.0.1 might fail with EADDRNOTAVAIL.  Binding to 0.0.0.0 is
+		# unnecessary on non-Windows systems.
 		#
 		# XXX A port available now may become unavailable by the time we start
 		# the postmaster.
 		if ($found == 1)
 		{
-			foreach my $addr (qw(127.0.0.1 0.0.0.0),
-				$use_tcp ? qw(127.0.0.2 127.0.0.3) : ())
+			foreach my $addr (qw(127.0.0.1),
+				$use_tcp ? qw(127.0.0.2 127.0.0.3 0.0.0.0) : ())
 			{
 				can_bind($addr, $port) or $found = 0;
 			}
