@@ -13,7 +13,7 @@ use warnings;
 use Config;
 use Cwd;
 use Exporter 'import';
-use Fcntl qw(:mode :seek);
+use Fcntl qw(:mode);
 use File::Basename;
 use File::Find;
 use File::Spec;
@@ -62,42 +62,21 @@ BEGIN
 	delete $ENV{LC_ALL};
 	$ENV{LC_MESSAGES} = 'C';
 
-	# This list should be kept in sync with pg_regress.c.
-	my @envkeys = qw (
-	  PGCLIENTENCODING
-	  PGCONNECT_TIMEOUT
-	  PGDATA
-	  PGDATABASE
-	  PGGSSLIB
-	  PGHOSTADDR
-	  PGKRBSRVNAME
-	  PGPASSFILE
-	  PGPASSWORD
-	  PGREQUIREPEER
-	  PGREQUIRESSL
-	  PGSERVICE
-	  PGSERVICEFILE
-	  PGSSLCERT
-	  PGSSLCRL
-	  PGSSLKEY
-	  PGSSLMODE
-	  PGSSLROOTCERT
-	  PGTARGETSESSIONATTRS
-	  PGUSER
-	  PGPORT
-	  PGHOST
-	);
-	delete @ENV{@envkeys};
+	delete $ENV{PGCONNECT_TIMEOUT};
+	delete $ENV{PGDATA};
+	delete $ENV{PGDATABASE};
+	delete $ENV{PGHOSTADDR};
+	delete $ENV{PGREQUIRESSL};
+	delete $ENV{PGSERVICE};
+	delete $ENV{PGSSLMODE};
+	delete $ENV{PGUSER};
+	delete $ENV{PGPORT};
+	delete $ENV{PGHOST};
 
 	$ENV{PGAPPNAME} = basename($0);
 
 	# Must be set early
 	$windows_os = $Config{osname} eq 'MSWin32' || $Config{osname} eq 'msys';
-	if ($windows_os)
-	{
-		require Win32API::File;
-		Win32API::File->import(qw(createFile OsFHandleOpen CloseHandle));
-	}
 }
 
 INIT
@@ -150,13 +129,8 @@ INIT
 END
 {
 
-	# Test files have several ways of causing prove_check to fail:
-	# 1. Exit with a non-zero status.
-	# 2. Call ok(0) or similar, indicating that a constituent test failed.
-	# 3. Deviate from the planned number of tests.
-	#
-	# Preserve temporary directories after (1) and after (2).
-	$File::Temp::KEEP_ALL = 1 unless $? == 0 && all_tests_passing();
+	# Preserve temporary directory for this test on failure
+	$File::Temp::KEEP_ALL = 1 unless all_tests_passing();
 }
 
 sub all_tests_passing
@@ -190,60 +164,22 @@ sub tempdir_short
 	return File::Temp::tempdir(CLEANUP => 1);
 }
 
-# Translate a Perl file name to a host file name.  Currently, this is a no-op
-# except for the case of Perl=msys and host=mingw32.  The subject need not
-# exist, but its parent directory must exist.
-sub perl2host
-{
-	my ($subject) = @_;
-	return $subject unless $Config{osname} eq 'msys';
-	my $here = cwd;
-	my $leaf;
-	if (chdir $subject)
-	{
-		$leaf = '';
-	}
-	else
-	{
-		$leaf = '/' . basename $subject;
-		my $parent = dirname $subject;
-		chdir $parent or die "could not chdir \"$parent\": $!";
-	}
-
-	# this odd way of calling 'pwd -W' is the only way that seems to work.
-	my $dir = qx{sh -c "pwd -W"};
-	chomp $dir;
-	chdir $here;
-	return $dir . $leaf;
-}
-
-# For backward compatibility only.
+# Return the real directory for a virtual path directory under msys.
+# The directory  must exist. If it's not an existing directory or we're
+# not under msys, return the input argument unchanged.
 sub real_dir
 {
-	return perl2host(@_);
-}
+	my $dir = "$_[0]";
+	return $dir unless -d $dir;
+	return $dir unless $Config{osname} eq 'msys';
+	my $here = cwd;
+	chdir $dir;
 
-=pod
-
-=item has_wal_read_bug()
-
-Returns true if $tmp_check is subject to a sparc64+ext4 bug that causes WAL
-readers to see zeros if another process simultaneously wrote the same offsets.
-Consult this in tests that fail frequently on affected configurations.  The
-bug has made streaming standbys fail to advance, reporting corrupt WAL.  It
-has made COMMIT PREPARED fail with "could not read two-phase state from WAL".
-Non-WAL PostgreSQL reads haven't been affected, likely because those readers
-and writers have buffering systems in common.  See
-https://postgr.es/m/20220116210241.GC756210@rfd.leadboat.com for details.
-
-=cut
-
-sub has_wal_read_bug
-{
-	return
-	     $Config{osname} eq 'linux'
-	  && $Config{archname} =~ /^sparc/
-	  && !run_log([ qw(df -x ext4), $tmp_check ], '>', '/dev/null', '2>&1');
+	# this odd way of calling 'pwd -W' is the only way that seems to work.
+	$dir = qx{sh -c "pwd -W"};
+	chomp $dir;
+	chdir $here;
+	return $dir;
 }
 
 sub system_log
@@ -292,37 +228,13 @@ sub slurp_dir
 
 sub slurp_file
 {
-	my ($filename, $offset) = @_;
+	my ($filename) = @_;
 	local $/;
-	my $contents;
-	my $fh;
-
-	# On windows open file using win32 APIs, to allow us to set the
-	# FILE_SHARE_DELETE flag ("d" below), otherwise other accesses to the file
-	# may fail.
-	if ($Config{osname} ne 'MSWin32')
-	{
-		open($fh, '<', $filename)
-		  or die "could not read \"$filename\": $!";
-	}
-	else
-	{
-		my $fHandle = createFile($filename, "r", "rwd")
-		  or die "could not open \"$filename\": $^E";
-		OsFHandleOpen($fh = IO::Handle->new(), $fHandle, 'r')
-		  or die "could not read \"$filename\": $^E\n";
-	}
-
-	if (defined($offset))
-	{
-		seek($fh, $offset, SEEK_SET)
-		  or die "could not seek \"$filename\": $!";
-	}
-
-	$contents = <$fh>;
-	close $fh;
-
-	$contents =~ s/\r\n/\n/g if $Config{osname} eq 'msys';
+	open(my $in, '<', $filename)
+	  or die "could not read \"$filename\": $!";
+	my $contents = <$in>;
+	close $in;
+	$contents =~ s/\r//g if $Config{osname} eq 'msys';
 	return $contents;
 }
 
@@ -455,7 +367,6 @@ sub check_pg_config
 	  \$stdout, '2>', \$stderr
 	  or die "could not execute pg_config";
 	chomp($stdout);
-	$stdout =~ s/\r$//;
 
 	open my $pg_config_h, '<', "$stdout/pg_config.h" or die "$!";
 	my $match = (grep { /^$regexp/ } <$pg_config_h>);
@@ -551,7 +462,6 @@ sub command_like
 	my $result = IPC::Run::run $cmd, '>', \$stdout, '2>', \$stderr;
 	ok($result, "$test_name: exit code 0");
 	is($stderr, '', "$test_name: no stderr");
-	$stdout =~ s/\r\n/\n/g if $Config{osname} eq 'msys';
 	like($stdout, $expected_stdout, "$test_name: matches");
 	return;
 }
@@ -583,7 +493,6 @@ sub command_fails_like
 	print("# Running: " . join(" ", @{$cmd}) . "\n");
 	my $result = IPC::Run::run $cmd, '>', \$stdout, '2>', \$stderr;
 	ok(!$result, "$test_name: exit code not 0");
-	$stderr =~ s/\r\n/\n/g if $Config{osname} eq 'msys';
 	like($stderr, $expected_stderr, "$test_name: matches");
 	return;
 }
@@ -610,8 +519,6 @@ sub command_checks_all
 	  if $ret & 127;
 	$ret = $ret >> 8;
 
-	foreach ($stderr, $stdout) { s/\r\n/\n/g if $Config{osname} eq 'msys'; }
-
 	# check status
 	ok($ret == $expected_ret,
 		"$test_name status (got $ret vs expected $expected_ret)");
